{
  lib,
  sources,
  pkgs,
  ...
}:
let
  inherit (lib)
    concatStringsSep
    mapAttrs
    mkOption
    ;
  inherit (lib.types)
    attrsOf
    bool
    enum
    listOf
    nullOr
    path
    str
    submodule
    unspecified
    ;

  patched_lix =
    let
      fetchTree-fix = pkgs.fetchurl {
        url = "https://gerrit.lix.systems/changes/lix~2399/revisions/1/patch?download";
        hash = "sha256-ecZOnpQmUYY2uSLwrt8QHkeLW+L5RZWVvi58h4GkGEY=";
        name = "fetchTree-fix";
        downloadToTemp = true;
        postFetch = ''
          base64 -d $downloadedFile > $out
        '';
      };
      editline-unpatch = pkgs.fetchurl {
        url = "https://gerrit.lix.systems/changes/lix~2865/revisions/1/patch?download";
        hash = "sha256-IsJuVCZcQZ9Md4+c44RxAYCDkKS/3CAZ2neCCMd/ueg=";
        name = "editline-unpatch";
        downloadToTemp = true;
        postFetch = ''
          base64 -d $downloadedFile > $out
        '';
      };
    in
    pkgs.applyPatches {
      src = sources.lix;
      name = "lix";
      patches = [
        fetchTree-fix
        editline-unpatch
      ];
    };

  default_overlay_paths = [
    "${sources.kat-pkgs}/overlay.nix"
    (pkgs.writeText "lix-overlay.nix" ''
      import "${sources.lix-overlay}/overlay.nix" { lix = ${patched_lix}; }
    '')
  ];

  machine_meta = {
    options = {
      version = mkOption {
        type = enum [
          "stable"
          "unstable"
        ];
      };
      patches = mkOption {
        type = listOf path;
        default = [ ];
      };
      overlay-paths = mkOption {
        type = listOf path;
        default = [ ];
        # /!\ Take care of imported files
      };

      wg-key = mkOption {
        type = nullOr str;
        default = null;
      };
      vpn-ip4 = mkOption {
        type = nullOr str;
        default = null;
      };
      vpn-hub = mkOption {
        type = bool;
        default = false;
        description = ''
          Peering with a hub should give access to all subnets,
          while hubs are the only points with multiple peers.
          Non hub peers with all hub, and vice-versa.
          TODO: multiple hubs ?
        '';
      };
      subnets = mkOption {
        type = listOf str;
        default = [ ];
      };
      fqdn = mkOption {
        type = nullOr str;
        default = null;
      };
    };
  };
  node_meta = cfg: rec {
    inherit (cfg)
      version
      wg-key
      vpn-ip4
      vpn-hub
      subnets
      fqdn
      ;
    patches = [
      ./nginx-fallback.patch
      ./ocamlPackagesExtentions.patch
    ] ++ cfg.patches;
    overlay-paths = default_overlay_paths ++ cfg.overlay-paths;
    nixpkgs = pkgs.applyPatches {
      src = sources."nixpkgs-${version}";
      name = "nixpkgs-${version}-patched";
      inherit patches;
    };
    nixpkgs-overlays = pkgs.writeText "nixpkgs-overlays.nix" ''
      [
        ${concatStringsSep "\n  " (map (p: "(import ${p})") overlay-paths)}
      ]
    '';
    home-manager = "${sources."home-manager-${version}"}/nixos";
    nixvim = import sources."nixvim-${version}";
  };
in
{
  options = {
    machines = mkOption {
      type = attrsOf (submodule machine_meta);
      apply = mapAttrs (_: node_meta);
    };
    general_overlays = mkOption {
      type = listOf unspecified;
      readOnly = true;
    };
  };
  config.general_overlays = map import default_overlay_paths;
}
